

# cy2madfile
# -----------------------------------------------------------------------------
# Update: 18/11/2013
# Created: 12/07/2013
# Function to create plane files with the necessary information to run MAD or
# to run PennCNV with them throw a cohort of cyhd.cychp.txt files generated
# from cy2apt throw an original CEL cohort from CytoScanHD.


# library(multicore)
# s <- sapply(list.files("RR", full.names=T), source)
# cychp.files="cy750.apt/"; output.name="rawData/"; output.type = "mad"
# cychp.attime = 8; quantile.bottom = 0.25; quantile.top = 0.75
# fill.24 = TRUE; verbose = TRUE

# madfile1.p <- read.table("rawData/12A030", header=TRUE, sep="\t")
# madfile2.p <- read.table("rawData/12A220", header=TRUE, sep="\t")
# madfile3.p <- read.table("rawData/1E094_L", header=TRUE, sep="\t")

cy2madfile <- function(cychp.files, output.name, output.type = "mad",
    cychp.attime = 4, quantile.bottom = 0.25, quantile.top = 0.75,
    fill.24 = TRUE, verbose = FALSE) {

    if(!CheckPython()) {
      screen_cyhd_mad(TRUE, TRUE, "This function required python and python-pandas. Please install them before run this function.")
    }
  
	SetCores(cychp.attime)
	
    # Check the directory. If empty we assume actual directory.
    if (cychp.files == "") {
        cychp.files = "./"
    }

    # Getting the full names of input files
    files.complete <- list.files(cychp.files, pattern = ".cyhd.cychp.txt",  full.names = TRUE);
    if (length(files.complete) == 0) {
        screen_cyhd_mad(TRUE, TRUE, "No 'cychp' files in folder '",
            cychp.files, "'.")
    }

    # Checking output type
    output.type <- value(output.type, c("mad", "penncnv"))
    if (is.na(output.type)) {
        screen_cyhd_mad(TRUE, TRUE,
            "Argument 'output.type' must be 'mad' or 'penncnv'.")
    }

    # Checking output name
    if (is.na(output.name) | output.name == '') {
        screen_cyhd_mad(TRUE, TRUE,
            "Argument 'output.name' must be filled.")
    }
    # /Checking input arguments

    # Generating the temp directory's name
    output.temp <- RandomName(prefix="tmp.cy.mad.")
    # /Generating the temp directory's name

    # Showing the input arguments
    m <- paste0( "\n--------------------------------------------------\n",
        " - Input:        ", cychp.files, "\n",
        " - Temp path:    ", output.temp, "\n",
        " - cychp x time: ", cychp.attime, "\n",
        " - Output:       ", output.type );
    m <- paste0( m, "\n--------------------------------------------------" )
    screen_cyhd_mad(verbose, FALSE, m)
    # /Showing the input arguments

    # Clean directories (output and temp)
    unlink(output.name, recursive=TRUE, force=TRUE)
    unlink(output.temp, recursive=TRUE, force=TRUE)
    dir.create(output.temp, showWarnings=FALSE)
    dir.create(output.name, showWarnings=FALSE)
    # /Clean directories
	
    data <- plapply(X=1:length(files.complete), FUN=CytoScanPreprocess,
        cychp.complete=files.complete, output.path=output.name, 
		output.temp=output.temp, bt=quantile.bottom, tt=quantile.top, 
		save=(output.type == "mad"), fill.24=fill.24, verbose=verbose)#,
        #mc.cores=cychp.attime)

    if (output.type == "penncnv") {
        unlink(output.name, recursive=TRUE, force=TRUE)
        screen_cyhd_mad(verbose, FALSE, "Number of files: ", length(data))

        f <- Reduce(intersect, lapply(data, function(x) { return(x$Name) }))

        result <- data[[1]];
        result <- result[result$Name %in% f, ]
        result <- result[!duplicated(result$Name), ]

        cel.name <- gsub(".cyhd.cychp.txt", "", basename(files.complete[1]))
        names <- c("Name", "Chr", "Position", paste0(cel.name, ".Log R Ratio"),
            paste0(cel.name, ".GType"), paste0(cel.name, ".B Allele Freq"));

        screen_cyhd_mad(verbose, FALSE, "Flatting files to penncnv.");
        for(nf in 2:length(data)) {
            screen_cyhd_mad(verbose, FALSE, "Flatting file ", nf)
            nn <- data[[nf]]
            nn <- nn[nn$Name %in% f, ]
            nn <- nn[!duplicated(nn$Name), ]

            result <- cbind(result, nn$Log.R.Ratio, nn$GType, nn$B.Allele.Freq)

            cel.name <- gsub(".cyhd.cychp.txt", "",
            basename(files.complete[nf ]));
            names <- c(names, paste0(cel.name, ".GType"),
                paste0(cel.name, ".Log R Ratio" ),
                paste0(cel.name, ".B Allele Freq"));
        }

        colnames(result) <- names;

        screen_cyhd_mad(verbose, FALSE, "Saving file '", output.name,
            ".penncnv'.");
        write.table( esult, file=paste0(output.name, ".penncnv"),
            quote=FALSE, row.names=FALSE, sep="\t", na="NA");
    }

    if (output.type == "mad") {
        #noncommon <- outersect.list.nx(lapply(data,
        #    function(x){ as.character(x[,1]) }))
        common <- intersect.list(data)
        message("len(common):", length(common))
        #save(common, "common.Rdata")
        #message("save(common)")

        #data.flat <- data[[1]]
        #for (ii in 2:length(data)) {
        #    data.flat <- rbind(data.flat, data[[ii]])
        #}

        #data.flat <- data.flat[data.flat$Name %in% common, ]
        #rm("common")
		
        ds <- plapply(X=list.files(output.name, full.names=TRUE),
            FUN=CytoScanPostprocess, probes.common=common, verbose=verbose)#,
            #mc.cores=cychp.attime, verbose)
        rm("ds")
    }

    unlink(output.temp, recursive=TRUE, force=TRUE);
    return(NA);
}
