

# CytoScanPreprocess
# -----------------------------------------------------------------------------
# Update: 18/11/2013
# Created: 12/07/2013

CytoScanProcess <- function( genotype.set, copynumber.set, allelepeaks.set,
    bt, tt, name.full, name.single, output.name, save, fill.24, output.temp,
    verbose ) {

    screen_cyhd_mad(verbose, FALSE, "Loading data for individual '",
        name.single, "'.");

    gds_f <- tryCatch( {
            read.delim( genotype.set, header=TRUE, as.is=TRUE )
        }, warnning=function(w) {
            screen_cyhd_mad(TRUE, TRUE,
                "Invalid file provided in 'genotypeset'.\n", w)
        }, error=function(e) {
            screen_cyhd_mad(TRUE, TRUE,
                "Invalid file provided in 'genotypeset'.\n", e)
        }
    )
    apds_f <- tryCatch( {
            read.delim(allelepeaks.set, sep="\t", header=TRUE, as.is=TRUE)
        }, warnning=function(w) {
            screen_cyhd_mad(TRUE, TRUE,
                "Invalid file provided in 'genotypeset'.\n", w)
        }, error=function(e) {
            screen_cyhd_mad(TRUE, TRUE,
                "Invalid file provided in 'genotypeset'.\n", e);
        }
    )
    cnds_f <- tryCatch( {
            read.delim(copynumber.set, header=TRUE, as.is=TRUE)
        }, warnning=function(w) {
            screen_cyhd_mad(TRUE, TRUE,
                "Invalid file provided in 'copynumberset'.", w)
        }, error=function(e) {
            screen_cyhd_mad(TRUE, TRUE,
                "Invalid file provided in 'copynumberset'.", e)
        }
    )

    rownames(gds_f) <- gds_f$ProbeSetName
    rownames(apds_f) <- apds_f$ProbeSetName
    rownames(cnds_f) <- cnds_f$ProbeSetName

    screen_cyhd_mad(verbose, FALSE, "Quantile filtering for individual '",
        name.single ,"'.");

    n <- quantile(apds_f$AllelePeaks0, c(bt, tt), rm.na=TRUE)
    apds_f$AllelePeaks0[apds_f$AllelePeaks0 < n[1]] <- n[1]
    apds_f$AllelePeaks0[apds_f$AllelePeaks0 > n[2]] <- n[2]

    ap <- (apds_f$AllelePeaks0 - n[1] ) / (n[2] - n[1])
    ap <- 1 - ap;

    screen_cyhd_mad(verbose, FALSE, "Creating data.frame for individual '",
        name.single, "'.");

    name <- file.path(output.name, name.single, fsep=.Platform$file.sep)

    madfile <- data.frame(cnds_f[rownames(apds_f), "ProbeSetName"],
        cnds_f[rownames(apds_f), "Chromosome"],
        cnds_f[rownames(apds_f), "Position"],
        cnds_f[rownames(apds_f), "Log2Ratio"], ap,
        gds_f[rownames(apds_f), "Call"])
    colnames(madfile) <- c("Name", "Chr", "Position", "Log.R.Ratio",
        "B.Allele.Freq", "GType")
    screen_cyhd_mad(verbose, FALSE, "data.frame for individual '", name,
        "' was created." );


    # save(madfile, file="madfile.Rdata")
    # save(cnds_f, file="cnds_f.Rdata")
    # save(gds_f, file="gnds_f.Rdata")
    # save(apds_f, file="apds_f.Rdata")
    # save(ap, file="ap.Rdata")

    screen_cyhd_mad(verbose, FALSE, "Cleaning temporal objects.")
    rm('ap', 'apds_f', 'gds_f', 'cnds_f')

    if (fill.24) {
        screen_cyhd_mad( verbose, FALSE, "Filling chromosome 24 with ",
            "dummy data from chromosome 23 for '", name.single, "'.")
        lny <- round(sum(madfile$Chr == "X") / 3)
        cn <- madfile[madfile$Chr == "X", ][1:lny, ]
        cn[ , "Chr"] <- "Y"
        cn[ , "Name"] <- unlist(sapply(as.character(cn$Name),
            function(x){ paste0("Y-", x) } ))
        madfile <- rbind(madfile, cn)
    }

    screen_cyhd_mad(verbose, FALSE, "Converting 'NC' to 'NA'.")
    madfile$GType[madfile$GType=="NC"] <- NA

    if (save) {
        screen_cyhd_mad( verbose, FALSE, "Writing file for individual '",
            name.single, "." )
        write.table(madfile, file=name, quote=FALSE, row.names=FALSE,
            sep="\t", na="NA")

        probes <- as.character(madfile$Name)
        rm('madfile')

        return(probes)
    } else {
        return(madfile)
    }
}
