
GWProcess <- function(cel.list, cel.genome, cel.attime, output.path,
    output.temp, output.type, cdfname, crlmm.oc, verbose) {

    ldPath(output.temp);

    screen_gw_mad(verbose, FALSE, "Loading CEL files.")
    cnSet <- tryCatch( {
        crlmm::constructAffyCNSet(cel.list, batch=rep("1", length(cel.list)),
            cdfName=cdfname, genome=cel.genome)
        }, warnning=function(w) {
            screen_gw_mad(verbose, TRUE,
                "Warning during \"cnSet\" construction.\n", w)
        }, error=function(e) {
            screen_gw_mad(verbose, TRUE,
                "Error during \"cnSet\" construction.\n", e)
        }
    )

    screen_gw_mad(verbose, FALSE, "Performing CNRMA.")
    tryCatch( {
        crlmm::cnrmaAffy(cnSet)
        }, warnning=function(w) {
            screen_gw_mad(verbose, TRUE,
                "Warning performing \"CNRMA\".\n", w)
        }, error=function(e) {
            screen_gw_mad(verbose, TRUE,
                "Error performing \"CNRMA\".\n", e )
        }
    )

    screen_gw_mad(verbose, FALSE, "Performing SNPRMA.")
    tryCatch( {
        crlmm::snprmaAffy(cnSet)
        },  warnning=function(w) {
            screen_gw_mad(verbose, TRUE, "Warning performing \"SNPRMA\".\n", w)
        }, error=function(e) {
            screen_gw_mad(verbose, TRUE, "Error performing \"SNPRMA\".\n", e)
        }
    )

    screen_gw_mad(verbose, FALSE, "Genotyping.")
    tryCatch( {
        crlmm::genotypeAffy( cnSet, gender=NULL )
        }, warnning=function(w) {
            screen_gw_mad(verbose, TRUE, "Warning on genotyping.\n", w)
        }, error=function(e) {
            screen_gw_mad(verbose, TRUE, "Error on genotyping.\n", e)
        }
    )

    screen_gw_mad(verbose, FALSE, "Performing CRLMM.")
    tryCatch( {
        crlmm::crlmmCopynumber(cnSet, fit.linearModel=FALSE)
        }, warnning=function(w) {
            screen_gw_mad(verbose, TRUE, "Warning performing \"CRLMM\".\n", w)
        }, error=function(e) {
            screen_gw_mad(verbose, TRUE, "Error performing \"CRLMM\".\n", e)
        }
    )

    screen_gw_mad(verbose, FALSE, "Coercing to BafLrrSetList.")
    oligoList <- tryCatch( {
        BafLrrSetList( cnSet )
        }, warnning=function(w) {
            screen_gw_mad(verbose, TRUE,
                "Warning on coercing to BafLrrSetList.\n", w )
        }, error=function(e) {
            screen_gw_mad(verbose, TRUE,
                "Error on coercing to BafLrrSetList.\n", e)
        }
    )

    screen_gw_mad(verbose, FALSE, "Creating BAF/LRR/calls container.")
    res <- list()
    res$calls <- calls(cnSet)
    res$calls <- res$calls[grep("SNP", rownames(res$calls)), ]
    b <- baf(oligoList)
    l <- lrr(oligoList)
    p <- position(oligoList)
    n <- featureNames(oligoList)

    flatting <- function(ii, baf_lrr_list) {
        ds <- ffdf(baf_lrr_list[[ii]],
            ff(ii, length=length(rownames(baf_lrr_list[[ii]]))))
        rownames(ds) <- rownames(baf_lrr_list[[ii]])
        colnames(ds) <- c(colnames(baf_lrr_list[[ii]]), "chr")
        return(ds);
    }

    b <- sapply(1:length(b) , flatting, b)
    screen_gw_mad(verbose, FALSE, "Filtering BAF.");
    res$baf <- data.frame(b[[1]])
    for(ii in 2:length(b)) {
        res$baf <- rbind(res$baf, data.frame(b[[ii]]))
    }
    res$baf <- res$baf[rownames(res$baf) %in% rownames(res$calls), ]
    rm("b");

    screen_gw_mad(verbose, FALSE, "Filtering LRR.")
    l <- sapply(1:length(l), flatting, l);
    res$lrr <- data.frame(l[[1]])
    for(ii in 2:length(l)) {
        res$lrr <- rbind(res$lrr, data.frame(l[[ii]]))
    }
    res$lrr <- res$lrr[rownames(res$lrr) %in% rownames(res$calls), ]
    rm("l");

    screen_gw_mad(verbose, FALSE, "Adding position and name.");
    res$pos <- vector()
    res$names <- vector()
    for(ii in 1:length(p)) {
        res$pos <- c(res$pos, p[[ii]])
        res$names <- c(res$names, n[[ii]])
    }
    names(res$pos) <- res$names
    res$names <- res$names[res$names %in% rownames(res$calls)]
    res$pos <- res$pos[names(res$pos) %in% rownames(res$calls)]
    rm("p", "n")

    screen_gw_mad(verbose, FALSE, "Naming calls.")
    c <- res$calls;
    l <- res$lrr;
    b <- res$baf;
    for(ii in 1:ncol(res$calls)) {
        res$calls[ , ii] <- c("AA", "AB", "BB")[c[ , ii]]
        res$lrr[ , ii] <- l[ , ii]/100
        res$baf[ , ii] <- normalize_baf(b[ , ii]/1000,
            res$calls[res$names , ii])
    }
    rm("c", "l", "b")

    res$calls <- res$calls[res$names, ]
    names(res) <- c("calls", "baf", "lrr", "pos", "names")

    return(res);
}
