#!/usr/bin/python2.7
# encoding: utf-8

import argparse
import os
import pandas
import sys


def main():

    # Step 1: Parsing arguments
    # ----------------------------------------------------------------------------------------
    # Here we got the program's arguments and its values.

    parser = argparse.ArgumentParser( description='Filter process for CNDS and GNDS given a APDS for CytoScan process with affy2sv.' )
    parser.add_argument( '--cnds', dest='cnds_file', help='Path to CNDS (CopyNumber) file.' )
    parser.add_argument( '--gnds', dest='gnds_file', help='Path to GNDS (genotype) file.' )
    parser.add_argument( '--apds', dest='apds_file', help='Path to APDS (AllelicPeack) file.' )
    parser.add_argument( '--out',  dest='output_path', help='Path were the output files will be stored.' )

    args = parser.parse_args()


    if not os.path.isfile( args.cnds_file ) or not os.path.isfile( args.gnds_file ) or not os.path.isfile( args.apds_file ):
        print '[ERROR]: Some of the given arguments is not a file.'
        sys.exit( 1 )

    if not os.path.isdir( args.output_path ):
        print '[ERROR]: Provided output path is not a directory.'
        sys.exit( 1 )

    # Step 2
    # ----------------------------------------------------------------------------------------
    # Load the files into pandas and filter using '.ix'. Then store the result.

    apds = pandas.read_csv( args.apds_file, sep='\t', index_col=0 )
    cnds = pandas.read_csv( args.cnds_file, sep='\t', index_col=0 )
    gnds = pandas.read_csv( args.gnds_file, sep='\t', index_col=8 )

    nn = apds.index

    cnds = cnds.ix[ nn ]
    gnds = gnds.ix[ nn ]

    cnds_n = os.path.basename( args.cnds_file )
    cnds_n = os.path.splitext( cnds_n )[ 0 ] + '.f.txt'

    gnds_n = os.path.basename( args.gnds_file )
    gnds_n = os.path.splitext( gnds_n )[ 0 ] + '.f.txt'

    cnds.to_csv( os.path.join( args.output_path, cnds_n ), sep='\t', nanRep='nan' )
    gnds.to_csv( os.path.join( args.output_path, gnds_n ), sep='\t', nanRep='nan' )

if __name__ == '__main__':
    main()