#!/usr/bin/python2.7
# encoding: utf-8


import os
import sys
import getopt

# Headers
headers = {
    'cyhd': {
        'sds': 'Chromosome Display StartIndex MarkerCount MinSignal MaxSignal MedianCnState HomFrequency HetFrequency Mosaicism LOH MedianSignal',
        'cnds': 'ProbeSetName Chromosome Position Log2Ratio WeightedLog2Ratio SmoothSignal',
        'apds': 'ProbeSetName Chromosome Position AllelePeaks0 AllelePeaks1',
        'mabsds': 'Index SCAR',
        'cn_ds': 'SegmentID Chromosome StartPosition StopPosition MarkerCount MeanMarkerDistance State Confidence',
        'lohds': 'SegmentID Chromosome StartPosition StopPosition MarkerCount MeanMarkerDistance LOH Confidence',
        'cnnlohds': 'SegmentID Chromosome StartPosition StopPosition MarkerCount MeanMarkerDistance CNNeutralLOH Confidence',
        'genotype': 'Index Call Confidence ForcedCall ASignal BSignal SignalStrength Contrast',
    },
    'affy6': {},
}


# Function used to show user's help.
def show_help():
    '''Function used to show user's help.'''
    print 'Usage:'
    print 'python splitter.py -i cych.cychp.txt -o tables [-t cyhd]'
    print 'python splitter.py -i cnchp.txt -o tables [-t affy6]'
    print
    exit( 0 )


def mach( line, chtype ):
    for key in headers[ chtype ]:
        if line == headers[ chtype ][ key ]:
            return key
    return None


if __name__ == '__main__':
    # Step 1: Parsing arguments
    # ----------------------------------------------------------------------------------------
    # Here we got the program's arguments and its values.
    opts, _ = getopt.getopt( sys.argv[ 1: ], 'i:o:ht:', [ 'inputfile', 'outputfolder', 'help', 'type' ] )

    # Here we check for the aerguments and we store the necessary values.
    out_prefix = None
    input_file = None
    chtype = 'cyhd'
    for opt, arg in opts:
        if opt in ( '-i', '--inputfile' ):
            input_file = arg
        if opt in ( '-o', '--outputfolder' ):
            out_prefix = arg
        if opt in ( '-h', '--help' ):
            show_help()
        if opt in ( '-t', '--type' ):
            chtype = arg.lower()


    # Show error if no input file is detected.
    if input_file is None:
        print '[ERROR]: No input file detected.'
        exit( 1 )

    # Check the output folder. If it doesn't exists we create it.
    if out_prefix is None:
        out_prefix = ""
    else:
        if not os.path.exists( out_prefix ):
            os.mkdir( out_prefix )


    # Step 2
    # ----------------------------------------------------------------------------------------
    # The idea is to copy to different files the tables into the cydh.cycho.txt files. So we i
    # read all the lines and we detect when we enter into a tables looking for the '#' 
    # character.
    # When we enter into a table we create a new file and we copy the incoming lines until we 
    # find a new '#' character.
    with open( input_file, 'r' ) as fi:
        out = os.path.join( out_prefix, reduce( lambda x, y: x + '.' + y,  fi.name.split( '/' )[ -1 ].split( '.' )[ :-1 ] ) )
        write = False
        nt, p, = 1, '#'
        for line in fi:
            if p.startswith( '#' ) and not line.startswith( '#' ):
                write = True
                tn = mach( line.strip().replace( '\t', ' ' ), chtype )
                if tn is None:
                    fo = open( out + '.table' + str( nt ) + '.txt', 'w' )
                    nt += 1
                else:
                    fo = open( out + '.' + tn + '.txt', 'w' )
            if not p.startswith( '#' ) and line.startswith( '#' ):
                write = False
                fo.close()
            if write:
                fo.write( line )
            p = line

